package ru.dbotthepony.mc.otm.mixin;

import net.minecraft.CrashReport;
import net.minecraft.CrashReportCategory;
import net.minecraft.ReportedException;
import net.minecraft.world.entity.player.Inventory;
import net.minecraft.world.entity.player.Player;
import net.minecraft.world.item.Item;
import net.minecraft.world.item.ItemStack;
import net.minecraftforge.registries.ForgeRegistries;
import org.spongepowered.asm.mixin.Mixin;
import org.spongepowered.asm.mixin.Shadow;
import org.spongepowered.asm.mixin.injection.At;
import org.spongepowered.asm.mixin.injection.Inject;
import org.spongepowered.asm.mixin.injection.callback.CallbackInfoReturnable;
import ru.dbotthepony.mc.otm.capability.MatteryCapability;

@Mixin(Inventory.class)
public class MixinInventory {
	@Shadow Player player;

	@Inject(
		method = "add(ILnet/minecraft/world/item/ItemStack;)Z",
		at = @At("HEAD"),
		cancellable = true
	)
	private void add(int pSlot, ItemStack pStack, CallbackInfoReturnable<Boolean> hook) {
		if (pStack.isEmpty()) {
			return;
		}

		if (pSlot == -1) {
			this.player.getCapability(MatteryCapability.MATTERY_PLAYER).ifPresent(it -> {
				try {
					hook.setReturnValue(it.inventoryAddImpl(pStack));
				} catch (Throwable throwable) {
					CrashReport crashreport = CrashReport.forThrowable(throwable, "Adding item to inventory (Overdrive That Matters detour)");
					CrashReportCategory crashreportcategory = crashreport.addCategory("Item being added");
					crashreportcategory.setDetail("Registry Name", () -> String.valueOf(ForgeRegistries.ITEMS.getKey(pStack.getItem())));
					crashreportcategory.setDetail("Item Class", () -> pStack.getItem().getClass().getName());
					crashreportcategory.setDetail("Item ID", Item.getId(pStack.getItem()));
					crashreportcategory.setDetail("Item data", pStack.getDamageValue());
					crashreportcategory.setDetail("Item name", () -> pStack.getHoverName().getString());
					throw new ReportedException(crashreport);
				}
			});
		}
	}
}
