package ru.dbotthepony.mc.otm.data.condition

import com.google.gson.JsonDeserializationContext
import com.google.gson.JsonObject
import com.google.gson.JsonPrimitive
import com.google.gson.JsonSerializationContext
import com.google.gson.JsonSyntaxException
import net.minecraft.world.level.storage.loot.LootContext
import net.minecraft.world.level.storage.loot.Serializer
import net.minecraft.world.level.storage.loot.predicates.LootItemCondition
import net.minecraft.world.level.storage.loot.predicates.LootItemConditionType
import ru.dbotthepony.mc.otm.core.nbt.set
import ru.dbotthepony.mc.otm.core.set
import ru.dbotthepony.mc.otm.registry.MLootItemConditions

/**
 * u serious?
 */
data class ChanceCondition(val chance: Double) : LootItemCondition, LootItemCondition.Builder {
	init {
		require(chance in 0.0 .. 1.0) { "Invalid chance: $chance" }
	}

	override fun test(t: LootContext): Boolean {
		return t.random.nextDouble() < chance
	}

	override fun getType(): LootItemConditionType {
		return MLootItemConditions.CHANCE
	}

	override fun build(): LootItemCondition {
		return this
	}

	companion object : Serializer<ChanceCondition> {
		override fun serialize(p_79325_: JsonObject, p_79326_: ChanceCondition, p_79327_: JsonSerializationContext) {
			p_79325_["chance"] = JsonPrimitive(p_79326_.chance)
		}

		override fun deserialize(p_79323_: JsonObject, p_79324_: JsonDeserializationContext): ChanceCondition {
			return ChanceCondition(p_79323_["chance"]?.asDouble ?: throw JsonSyntaxException("Invalid chance json element"))
		}
	}
}
