package ru.dbotthepony.mc.otm.recipe

import com.google.gson.JsonObject
import net.minecraft.core.NonNullList
import net.minecraft.network.FriendlyByteBuf
import net.minecraft.resources.ResourceLocation
import net.minecraft.world.inventory.CraftingContainer
import net.minecraft.world.item.ItemStack
import net.minecraft.world.item.crafting.CraftingBookCategory
import net.minecraft.world.item.crafting.Ingredient
import net.minecraft.world.item.crafting.RecipeSerializer
import net.minecraft.world.item.crafting.ShapedRecipe
import net.minecraft.world.level.Level
import ru.dbotthepony.mc.otm.capability.matteryEnergy
import ru.dbotthepony.mc.otm.container.iterator
import ru.dbotthepony.mc.otm.container.stream
import ru.dbotthepony.mc.otm.core.filterNotNull

class EnergyContainerRecipe(
	id: ResourceLocation,
	group: String,
	category: CraftingBookCategory,
	width: Int,
	height: Int,
	ingredients: NonNullList<Ingredient>,
	result: ItemStack,
) : ShapedRecipe(id, group, category, width, height, ingredients, result) {
	constructor(parent: ShapedRecipe) : this(parent.id, parent.group, parent.category(), parent.width, parent.height, parent.ingredients, parent.resultItem)

	override fun assemble(container: CraftingContainer): ItemStack {
		val itemStack = super.assemble(container)

		val battery = container.stream()
			.filter { !it.isEmpty }
			.map { it.matteryEnergy }
			.filterNotNull()
			.findAny().orElse(null)

		if (battery != null) {
			itemStack.matteryEnergy?.batteryLevel = battery.batteryLevel
		}

		if (itemStack.isEnchantable) {
			for (it in container.iterator()) {
				if (!it.isEmpty && it.isEnchanted) {
					for ((key, value) in it.allEnchantments) {
						itemStack.enchant(key, value)
					}
				}
			}
		}

		return itemStack
	}

	override fun matches(container: CraftingContainer, level: Level): Boolean {
		return super.matches(container, level) && !container.stream().anyMatch { it.isDamaged }
	}

	override fun getSerializer(): RecipeSerializer<EnergyContainerRecipe> {
		return Companion
	}

	companion object : RecipeSerializer<EnergyContainerRecipe> {
		override fun fromJson(id: ResourceLocation, data: JsonObject): EnergyContainerRecipe {
			return EnergyContainerRecipe(Serializer.SHAPED_RECIPE.fromJson(id, data))
		}

		override fun fromNetwork(id: ResourceLocation, data: FriendlyByteBuf): EnergyContainerRecipe? {
			return Serializer.SHAPED_RECIPE.fromNetwork(id, data)?.let(::EnergyContainerRecipe)
		}

		override fun toNetwork(buff: FriendlyByteBuf, value: EnergyContainerRecipe) {
			Serializer.SHAPED_RECIPE.toNetwork(buff, value)
		}
	}
}
